/**
 * Copyright (c) 2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.artifactrepository.impl;

import java.util.Properties;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.util.Fractal;
import org.ow2.petals.extension.artifactrepository.api.ArtifactRepositoryService;
import org.ow2.petals.microkernel.api.extension.PetalsExtensionController;
import org.ow2.petals.microkernel.api.extension.exception.CreationExtensionException;
import org.ow2.petals.microkernel.api.server.FractalHelper;

/**
 * Controller of the Petals extension 'Artifact Repository'
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ExtensionController implements PetalsExtensionController {

    /**
     * Name of the property enabling/disabling the artifact repository.
     */
    private static final String ARTIFACTREPOSITORY = "petals.artifact-repository";

    /**
     * Default value of the property {@value #ARTIFACTREPOSITORY}.
     */
    private static final boolean DEFAULT_ARTIFACTREPOSITORY = true;

    /**
     * The name of the resources containing the Fractal composite definition
     * associated to the Petals ESB Artifact Repository
     */
    private static final String ARTIFACTREPOSITORY_COMPOSITE = "ArtifactRepository";

    /**
     * The name of the client interface of the configuration service of the
     * Petals ESB artifact repository composite
     */
    private final static String CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * {@inheritDoc}
     */
    public Component createFractalComponent(final Component extensionComposite)
            throws CreationExtensionException {

        try {
            // Create the Fractal component associated to the Petals ESB
            // Artifact Repository
            final Component extensionImpl = FractalHelper
                    .createNewComponent(ARTIFACTREPOSITORY_COMPOSITE);

            // Add the Petals ESB Artifact Repository component in the Extension
            // composite
            FractalHelper.addComponent(extensionImpl, extensionComposite, null);
            final NameController extensionNC = Fractal.getNameController(extensionImpl);
            extensionNC.setFcName(ArtifactRepositoryService.ARTIFACTREPOSITORY_COMPONENT_NAME);

            // Next we bind client interfaces of the Petals ESB Artifact
            // Repository to the right interfaces
            final ContentController extensionCC = Fractal.getContentController(extensionComposite);

            final BindingController extensionBC = Fractal.getBindingController(extensionImpl);

            // As the configuration component is not a sub-component of the
            // extension composite, we link the client interface "configuration"
            // of the artifact repository to the client interface
            // "configuration" of the extension composite.
            final Interface configurationServiceItf = (Interface) extensionCC
                    .getFcInternalInterface(PetalsExtensionController.CONFIGURATION_FRACTAL_ITF_NAME);
            extensionBC.bindFc(ExtensionController.CONFIGURATION_FRACTAL_ITF_NAME,
                    configurationServiceItf);

            return extensionImpl;
        } catch (final ADLException e) {
            throw new CreationExtensionException(ArtifactRepositoryService.EXTENSION_NAME, e);
        } catch (final NoSuchInterfaceException e) {
            throw new CreationExtensionException(ArtifactRepositoryService.EXTENSION_NAME, e);
        } catch (final IllegalContentException e) {
            throw new CreationExtensionException(ArtifactRepositoryService.EXTENSION_NAME, e);
        } catch (final IllegalLifeCycleException e) {
            throw new CreationExtensionException(ArtifactRepositoryService.EXTENSION_NAME, e);
        } catch (final IllegalBindingException e) {
            throw new CreationExtensionException(ArtifactRepositoryService.EXTENSION_NAME, e);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void removeFractalComponent() {
        // TODO Auto-generated method stub

    }

    /**
     * {@inheritDoc}
     */
    public String getExtensionName() {
        return ArtifactRepositoryService.EXTENSION_NAME;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isActivated(final Properties properties) {
        final String propertyValue = properties.getProperty(ARTIFACTREPOSITORY);
        final boolean isActivated;
        if (propertyValue != null) {
            isActivated = Boolean.parseBoolean(propertyValue);
        } else {
            isActivated = DEFAULT_ARTIFACTREPOSITORY;
        }

        return isActivated;
    }

    /**
     * {@inheritDoc}
     */
    public String[] getDependencies() {
        return null;
    }

}
