/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.artifactrepository.impl;

import java.io.File;
import java.util.List;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.extension.artifactrepository.api.Artifact;
import org.ow2.petals.extension.artifactrepository.api.ArtifactRepositoryService;
import org.ow2.petals.launcher.api.server.PetalsServer;
import org.ow2.petals.launcher.api.server.conf.ConfigurationProperties;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;

import com.ebmwebsourcing.easycommons.properties.PropertiesException;
import com.ebmwebsourcing.easycommons.properties.PropertiesHelper;

/**
 * The artifact repository service.
 * 
 * @author Christophe HAMERLING - EBM WebSourcing
 * 
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = ArtifactRepositoryService.class))
public class FractalArtifactRepositoryService implements ArtifactRepositoryService {

    /**
     * Name of the property defining the directory used as repository
     */
    public static final String ARTIFACT_REPOSITIRY_PATH_PROPERTY_NAME = "petals.artifact-repository.path";

    /**
     * Default value of the directory used as repository: ${
     * {@value PetalsServer#DATA_ROOT_DIRECTORY_PROPERTY_NAME}/artifacts
     */
    private static final String DEFAULT_PATH = "${"
            + ConfigurationProperties.DATA_ROOT_DIRECTORY_PROPERTY_NAME + "}" + File.separator
            + "artifacts";

    private ArtifactRepositoryService delegate;

    @Requires(name = "configuration", signature = ConfigurationService.class)
    private ConfigurationService configurationService;

    private LoggingUtil log;

    public FractalArtifactRepositoryService() {
        // NOP
    }

    @LifeCycle(on = LifeCycleType.START)
    public void start() throws PropertiesException {
        this.log = new LoggingUtil(java.util.logging.Logger.getLogger(ArtifactRepositoryService.COMPONENT_LOGGER_NAME));
        this.log.start();
        
        final ContainerConfiguration containerConfiguration = this.configurationService
                .getContainerConfiguration();
        final String repositoryDirectoryPropertyValue = containerConfiguration
                .getExtraConfiguration().get(
                ARTIFACT_REPOSITIRY_PATH_PROPERTY_NAME);
        final String repositoryDirectoryStr;
        if (repositoryDirectoryPropertyValue == null || repositoryDirectoryPropertyValue.isEmpty()) {
            repositoryDirectoryStr = PropertiesHelper.resolveString(
                    DEFAULT_PATH, this.configurationService.getServerProperties());
        } else {
            repositoryDirectoryStr = repositoryDirectoryPropertyValue;
        }

        this.delegate = new LocalArtifactRepositoryService(new File(repositoryDirectoryStr));

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() {
        this.log.start();
        this.delegate = null;
        this.log.end();
    }

    public List<Artifact> getComponents() {
        return delegate.getComponents();
    }

    public File getComponentsPath() {
        return delegate.getComponentsPath();
    }

    public File getRootPath() {
        return delegate.getRootPath();
    }

    public boolean addArtifact(Artifact artifact) throws PetalsException {
        return delegate.addArtifact(artifact);
    }

    public List<String> getComponentsName() {
        return delegate.getComponentsName();
    }

    public List<String> getServiceAssembliesName() {
        return delegate.getServiceAssembliesName();
    }

    public File getServiceAssemblyPath() {
        return delegate.getServiceAssemblyPath();
    }

    public List<String> getSharedLibrariesName() {
        return delegate.getSharedLibrariesName();
    }

    public File getSharedLibraryPath() {
        return delegate.getSharedLibraryPath();
    }

    public List<Artifact> getServiceAssemblies() {
        return delegate.getServiceAssemblies();
    }

    public List<Artifact> getSharedLibraries() {
        return delegate.getSharedLibraries();
    }

}
