/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.artifactrepository.impl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.List;

import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.extension.artifactrepository.api.Artifact;
import org.ow2.petals.extension.artifactrepository.api.ArtifactRepositoryService;

/**
 * @author Christophe HAMERLING - EBM WebSourcing
 */
public class LocalArtifactRepositoryTest {

    @Rule
    public TemporaryFolder temporaryFolder = new TemporaryFolder();

    @Test
    public void testAddArtifacts() throws URISyntaxException, PetalsException {

        final File artifactRepositoryRootPath = this.temporaryFolder.newFolder("artifact");

        final ArtifactRepositoryService repositoryService = new LocalArtifactRepositoryService(
                artifactRepositoryRootPath);
        assertEquals(repositoryService.getComponentsPath().listFiles().length, 0);

        final URL componentURL = Thread.currentThread().getContextClassLoader()
                .getResource("packages/installation01.zip");
        final File component = new File(componentURL.toURI());
        assertTrue(component.exists());

        final Artifact artifact = new Artifact();
        artifact.setFile(component);
        boolean result = repositoryService.addArtifact(artifact);
        assertTrue(result);
        assertEquals(repositoryService.getComponentsPath().listFiles().length, 1);

        assertEquals(repositoryService.getServiceAssemblyPath().listFiles().length, 0);
        final URL saURL = Thread.currentThread().getContextClassLoader()
                .getResource("packages/sa.zip");
        final File sa = new File(saURL.toURI());
        artifact.setFile(sa);
        result = repositoryService.addArtifact(artifact);
        assertTrue(result);
        assertEquals(repositoryService.getServiceAssemblyPath().listFiles().length, 1);
    }

    @Test
    public void testGetComponent() throws URISyntaxException {

        final File artifactRepositoryRootPath = this.temporaryFolder.newFolder("artifact");

        final ArtifactRepositoryService repositoryService = new LocalArtifactRepositoryService(
                artifactRepositoryRootPath);
        assertEquals(repositoryService.getComponentsPath().listFiles().length, 0);

        final URL componentURL = Thread.currentThread().getContextClassLoader()
                .getResource("packages/installation01.zip");
        final File component = new File(componentURL.toURI());
        assertTrue(component.exists());

        final Artifact artifact = new Artifact();
        artifact.setFile(component);
        boolean result;
        try {
            result = repositoryService.addArtifact(artifact);
            assertTrue(result);
        } catch (final PetalsException e) {
            fail(e.getMessage());
        }

        final List<Artifact> components = repositoryService.getComponents();
        assertEquals(1, components.size());
        assertEquals("example-name", components.get(0).getName());

        final List<String> names = repositoryService.getComponentsName();
        assertEquals(1, names.size());
        assertEquals("example-name", names.get(0));
    }
}
